@@ -437,6 +437,15 @@ router:add("DELETE","/plugins/<id>/variables",function(path, data, vars, query)
   return Emu.api.hc3.restricted.delete(path,data)
 end)
 
+router:add("POST","/plugins/interfaces",function(path, data, vars, query) 
+  local dev = Emu.DIR[vars.id]
+  if dev then
+    if dev.device.isProxy then return Emu.api.hc3.restricted.delete(path,data) end
+    -- ToDo, add interfaces
+  end
+  return Emu.api.hc3.restricted.delete(path,data)
+end)
+
 router:add("POST", "/plugins/callUIEvent", function(path, data, vars, query)
   --return create_response({status = "ui_event_called"})
   local dev = Emu.DIR[data.deviceID]
@@ -532,9 +541,10 @@ router:add("POST", "/plugins/updateProperty", function(path, data, vars, query)
       -- Generate refreshState event
       if not dev.device.isProxy then
         Emu:refreshEvent('DevicePropertyUpdatedEvent',{
-          deviceId = id,
-          propertyName = prop,
+          id = id,
+          property = prop,
           newValue = value,
+          oldValue = dev.device.properties[prop]
         })
       end
       if dev.watches and dev.watches[prop] then
@@ -590,6 +600,16 @@ local function findFile(name,files)
   for i,f in ipairs(files) do if f.name == name then return f,i end end
 end
 
+router:add("GET", "/quickApp/<id>/info", function(path, data, vars, query)
+  local info = Emu:getQuickApp(vars.id)
+  return info,HTTP.OK
+end)
+
+router:add("GET", "/quickApp/info", function(path, data, vars, query)
+  local info = Emu:getQuickApps()
+  return info,HTTP.OK
+end)
+
 router:add("GET", "/quickApp/<id>/files", function(path, data, vars, query)
   local dev = Emu.DIR[vars.id]
   if not (dev and dev.files) then if Emu.offline then return nil,HTTP.NOT_FOUND else return nil,301 end end
@@ -611,6 +631,7 @@ router:add("POST", "/quickApp/<id>/files", function(path, data, vars, query)
 end)
 
 router:add("PUT", "/quickApp/<id>/files", function(path, data, vars, query)
+  print("PUTFILES")
   local dev = Emu.DIR[vars.id]
   if not dev then if Emu.offline then return nil,HTTP.NOT_FOUND else return nil,301 end end
   local files = dev.files or {}
