@@ -78,8 +78,9 @@ end
 
 local logStr = function(...) 
   local b = {} 
+  local formatOutput = fibaro.plua.formatOutput or tostring
   for _,e in ipairs({...}) do 
-    b[#b+1]=tostring(e) 
+    b[#b+1]=formatOutput(e)
   end 
   return table.concat(b," ")
 end
@@ -203,6 +204,7 @@ function fibaro.call(deviceId, action, ...)
     __assert_type(deviceId, "number")
     local arg = {...}
     local arg2 = #arg>0 and arg or nil
+    print(json.encodeFast(arg2))
     return api.post("/devices/"..deviceId.."/action/"..action, { args = arg2 })
   end
 end
