@@ -1,4 +1,4 @@
--- @module hc3emu2.quickapp
+-- @module fibaro.quickapp
 ---@description QuickApp core functionality for Plua
 ---@author Jan Gabrielsson
 ---@license MIT
@@ -252,6 +252,7 @@ function QuickAppBase:setVariable(name, value)
   if not found then
     table.insert(qvars, {name=name, value=value})
   end
+  self:updateProperty("quickAppVariables", qvars)
 end
 
 -- Gets a QuickApp variable value
@@ -389,15 +390,17 @@ end
 -- @param event - Event object containing action details
 function onAction(id,event) -- { deviceID = 1234, actionName = "test", args = {1,2,3} }
   --if Emu:DBGFLAG('onAction') then print("onAction: ", json.encode(event)) end
+  setTimeout(function()
   local self = plugin.mainQA
   ---@diagnostic disable-next-line: undefined-field
   if self.actionHandler then return self:actionHandler(event) end
   if event.deviceId == self.id then
-    return self:callAction(event.actionName, table.unpack(event.args))
+    return self:callAction(event.actionName, table.unpack(event.args or {}))
   elseif self.childDevices[event.deviceId] then
     return self.childDevices[event.deviceId]:callAction(event.actionName, table.unpack(event.args or {}))
   end
   self:error(fmt("Child with id:%s not found",id))
+end,0)
 end
 
 -- Global handler for UI events
@@ -416,7 +419,7 @@ function onUIEvent(id, event)
       fibaro.warning(__TAG,fmt("UI callback for %s %s not found.", event.elementName, event.eventType))
       return
     end
-    quickApp:callAction(action, event)
+    setTimeout(function() quickApp:callAction(action, event) end,0)
   else
     fibaro.warning(__TAG,fmt("UI callback for element %s not found.", event.elementName))
   end
@@ -489,16 +492,17 @@ function RefreshStateSubscriber:unsubscribe(subscription)
 end
 
 local listeners = {}
-function _PY.newRefreshStatesEvent(jsonevent)
-  --print("New jsonevent",jsonevent) -- jsonevent)
-  local event = json.decode(jsonevent)
-  for listener,_ in pairs(listeners) do
-    pcall(listener,event)
-  end
-end
+
 -- Starts the refresh state subscriber
 ---@diagnostic disable-next-line: undefined-field
 function RefreshStateSubscriber:run()
+  function _PY.newRefreshStatesEvent(jsonevent)
+  --print("New jsonevent",jsonevent) -- jsonevent)
+  local event = json.decode(jsonevent)
+  for l,_ in pairs(listeners) do
+    pcall(l,event)
+  end
+end
   fibaro.plua:startRefreshStatesPolling()
   listeners[self.handle] = true
 end
