@@ -36,14 +36,16 @@ local function findFirstLine(src)
   return first or 1,init
 end
 
-local function loadQAString(src,optionalDirectives) -- Load QA from string and run it
+local lfs = require("lfs")
+local function loadQAString(src,options) -- Load QA from string and run it
   local path = Emu.config.tempdir..createTempName(".lua")
+  lfs.mkdir(Emu.config.tempdir)
   local f = io.open(path,"w")
   assert(f,"Can't open file "..path)
   f:write(src)
   f:close()
   ---@diagnostic disable-next-line: need-check-nil
-  return Emu:installQuickAppFile(path)
+  return Emu:installQuickAppFile(path, options or {})
 end
 
 
@@ -57,6 +59,43 @@ local function arrayifyFqa(fqa)
   return fqa
 end
 
+-- Screen dimension and QuickApp window functions
+local function getScreenDimension()
+  local dims = _PY.get_screen_dimensions()
+  if dims then
+    return dims.width, dims.height
+  else
+    return 1920, 1080  -- Fallback
+  end
+end
+
+local function createQuickAppWindow(qaId, title, width, height, x, y, backgroundColor)
+  title = title or fmt("QuickApp %s", qaId)
+  width = width or 800
+  height = height or 600
+  x = x or 100
+  y = y or 100
+  backgroundColor = backgroundColor or ""
+  
+  local success = _PY.open_quickapp_window(qaId, title, width, height, x, y, backgroundColor)
+  if success then
+    Emu:DEBUG(fmt("Created QuickApp window for QA %s: %s (%dx%d at %d,%d)", qaId, title, width, height, x, y))
+    if backgroundColor ~= "" then
+      Emu:DEBUG(fmt("Applied background color: %s", backgroundColor))
+    end
+  else
+    Emu:WARNING(fmt("Failed to create QuickApp window for QA %s", qaId))
+  end
+  return success
+end
+
+local function setQuickAppWindowBackground(qaId, color)
+  -- For existing windows, we can't change the background color after creation
+  -- This function is kept for compatibility but will return false for existing windows
+  Emu:WARNING(fmt("Cannot change background color for existing QuickApp window %s. Use createQuickAppWindow with backgroundColor parameter instead.", qaId))
+  return false
+end
+
 local function uploadFQA(fqa)
   assert(type(fqa) == "table", "fqa must be a table")
   assert(fqa.name, "fqa must have a name")
@@ -69,7 +108,10 @@ local function uploadFQA(fqa)
   end
   assert(haveMain, "fqa must have a main file")
   arrayifyFqa(fqa)
-  local res,code = Emu.api.hc3.post("/quickApp/",json.encodeFast(fqa))
+  --fqa.files = {}
+  local encodedFQA = json.encodeFast(fqa)
+  --print("Uploading FQA:",encodedFQA)
+  local res,code = Emu.api.hc3.post("/quickApp/",encodedFQA)
   if not code or code > 201 then
     Emu:ERROR("Failed to upload FQA: "..tostring(code))
   else
@@ -78,12 +120,10 @@ local function uploadFQA(fqa)
   return res,code
 end
 
-local function getFQA(id) -- Creates FQA structure from installed QA
-  local dev = Emu.DIR[id]
-  assert(dev,"QuickApp not found, ID"..tostring(id))
-  local struct = dev.device
+local function info2FQA(info)
+  local struct = info.device
   local files = {}
-  for name,f in pairs(dev.files) do
+  for name,f in pairs(info.files) do
     if f.content == nil then f.content = Emu.lib.readFile(f.path) end
     files[#files+1] = {name=name, isMain=name=='main', isOpen=false, type='lua', content=f.content}
   end
@@ -110,6 +150,12 @@ local function getFQA(id) -- Creates FQA structure from installed QA
   return arrayifyFqa(struct)
 end
 
+local function getFQA(id) -- Creates FQA structure from installed QA
+  local dev = Emu.DIR[id]
+  assert(dev,"QuickApp not found, ID"..tostring(id))
+  return info2FQA(dev)
+end
+
 local function saveQA(id,fileName) -- Save installed QA to disk as .fqa  //Move to QA class
   local dev = Emu.devices[id]        
   fileName = fileName or dev.headers.save
@@ -132,10 +178,11 @@ local function saveQA(id,fileName) -- Save installed QA to disk as .fqa  //Move
   Emu:DEBUG("Saved QuickApp to %s",fileName)
 end
 
-local function loadQA(path,optionalHeaders,noRun)   -- Load QA from file and maybe run it
-  optionalHeaders = optionalHeaders or {}
-  optionalHeaders.norun = tostring(noRun==true) -- If noRun is true, don't run the QuickApp
-  local struct = Emu:installQuickAppFile(path,optionalHeaders)
+local function loadQA(path,options)   -- Load QA from file and maybe run it
+  options = options or {} 
+  options.headers = options.headers or {}
+  table.insert(options.headers,"norun:"..tostring(options.noRun==true)) -- If noRun is true, don't run the QuickApp
+  local struct = Emu:installQuickAppFile(path,options)
   return struct
 end
 
@@ -178,7 +225,7 @@ local function unpackFQAAux(id,fqa,path) -- Unpack fqa and save it to disk
   
   local qvars = props.quickAppVariables or {}
   for _,v in ipairs(qvars) do
-    pr:printf('--%%%%var:%s=%s',v.name,type(v.value)=='string' and '"'..v.value..'"' or v.value)
+    pr:printf('--%%%%var:%s=%s',v.name,type(v.value)=='string' and '"'..v.value:gsub('"','\\"')..'"' or v.value)
   end
   if id then pr:printf('--%%%%project:%s',id) end
   if props.quickAppUuid then pr:printf('--%%%%uid:%s',props.quickAppUuid) end
@@ -461,6 +508,11 @@ local function updateFile(fname)
   end
 end
 
+local function luaToFQA(code)
+    local info = Emu:createInfoFromContent("",code)
+    return info2FQA(info)
+end
+
 Emu.lib.createTempName = createTempName
 Emu.lib.findFirstLine = findFirstLine
 Emu.lib.loadQAString = loadQAString
@@ -473,4 +525,11 @@ Emu.lib.unpackFQAAux = unpackFQAAux
 Emu.lib.saveProject = saveProject
 Emu.lib.updateQA = updateQA
 Emu.lib.updateFile = updateFile
-Emu.lib.updateQAparts = updateQAparts
\ No newline at end of file
+Emu.lib.getScreenDimension = getScreenDimension
+Emu.lib.createQuickAppWindow = createQuickAppWindow
+Emu.lib.setQuickAppWindowBackground = setQuickAppWindowBackground
+Emu.lib.updateFile = updateFile
+Emu.lib.updateQAparts = updateQAparts
+Emu.lib.info2FQA = info2FQA
+Emu.lib.luaToFQA = luaToFQA
+Emu.lib.createTempName = createTempName
\ No newline at end of file
